// const exec = require('child-process-promise').exec
// // Этот скрипт работает in-place, то есть оптимизирует исходные файлы
// module.exports = (pngquantOptions = '--quality=30-50 -s1', zopflipngOptions = '-m') => imagePath => {
// 	// Сначала запускаем pngquant, который делает lossy сжатие с сокращением количества цветов в png изображении
// 	const pngquantCommand = `pngquant ${pngquantOptions} -f --ext .png "${imagePath}"`
// 	return exec(pngquantCommand).then(({ stdout, stderr }) => {
// 		if (stderr) {
// 			throw new Error(`stderr: ${stderr}`)
// 		}
// 		// Затем запускаем lossless zopflipng, который удаляет всю мета информацию и без потерь сжимает png изображение
// 		// zopflipng берёт файл полученный от pngquant и перезаписывает его
// 		const zopflipngCommand = `zopflipng ${zopflipngOptions} -y "${imagePath}" "${imagePath}"`
// 		return exec(zopflipngCommand).then(({ stdout, stderr }) => {
// 			if (stderr) {
// 				throw new Error(`stderr: ${stderr}`)
// 			}
// 			console.log(`stdout: ${stdout}`)
// 		})
// 	}).catch(err => {
// 		if (err) {
// 			console.log('Error!', err)
// 			return
// 		}
// 	})
// }
'use strict';

const execa = require('execa');

const isPng = require('is-png');

const isStream = require('is-stream');

const pngquant = require('pngquant-bin');

const ow = require('ow');

const optimizePng = (options = {}) => input => {
  const isBuffer = Buffer.isBuffer(input);

  if (!isBuffer && !isStream(input)) {
    return Promise.reject(new TypeError(`Expected a Buffer or Stream, got ${typeof input}`));
  }

  if (isBuffer && !isPng(input)) {
    return Promise.resolve(input);
  }

  const args = ['-'];

  if (typeof options.speed !== 'undefined') {
    ow(options.speed, ow.number.integer.inRange(1, 11));
    args.push('--speed', options.speed);
  }

  if (typeof options.strip !== 'undefined') {
    ow(options.strip, ow.boolean);
    args.push('--strip');
  }

  if (typeof options.quality !== 'undefined') {
    ow(options.quality, ow.array.length(2).ofType(ow.number.inRange(0, 1)));
    const [min, max] = options.quality;
    args.push('--quality', `${Math.round(min * 100)}-${Math.round(max * 100)}`);
  }

  if (typeof options.dithering !== 'undefined') {
    ow(options.dithering, ow.any(ow.number.inRange(0, 1), ow.boolean.false));

    if (typeof options.dithering === 'number') {
      args.push(`--floyd=${options.dithering}`);
    } else if (options.dithering === false) {
      args.push('--ordered');
    }
  }

  if (typeof options.posterize !== 'undefined') {
    ow(options.posterize, ow.number);
    args.push('--posterize', options.posterize);
  }

  if (typeof options.verbose !== 'undefined') {
    ow(options.verbose, ow.boolean);
    args.push('--verbose');
  }

  const subprocess = execa(pngquant, args, {
    encoding: null,
    maxBuffer: Infinity,
    input
  });
  const promise = subprocess.then(result => result.stdout) // eslint-disable-line promise/prefer-await-to-then
  .catch(error => {
    if (error.code === 99) {
      return input;
    }

    error.message = error.stderr || error.message;
    throw error;
  });
  subprocess.stdout.then = promise.then.bind(promise); // eslint-disable-line promise/prefer-await-to-then

  subprocess.stdout.catch = promise.catch.bind(promise);
  return subprocess.stdout;
};

module.exports = optimizePng;
module.exports.default = optimizePng;